;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Deadweight, by imamelia
;;
;; This is a recoded version of Schwa's deadweight sprite that the Deadweight Bro
;; throws.
;;
;; Uses first extra bit: NO
;;
;; $C2,x - bounce speed table index (set this in the spawning routine)
;; $187B,x - number of bounces the sprite will make
;; $160E,x - bounce speed table value counter
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; defines and tables
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

!WeightTileNum = $80		; the tile number that the sprite uses
!TimeToShake = $18		; time to shake the ground (and possibly stun the player) upon impact
!ImpactSFX = $091DFC		; sound effect to play when the sprite hits the ground

BounceInfoPtrs:			;
dw BounceSetting0		;
dw BounceSetting1		;
dw BounceSetting2		;
dw BounceSetting3		;
; You can add more pointers here, up to 7F.

BounceSetting0:			; bounce setting 0
db $04				; 4 bounces
db $B4,$B4,$B4,$B4		; same height each time
db $10,$10,$10,$10		; same X speed each time

BounceSetting1:			; bounce setting 1
db $04				; 4 bounces
db $B4,$C0,$B4,$A8		; different heights
db $10,$10,$10,$10		; same X speed each time

BounceSetting2:			; bounce setting 2
db $03				; 3 bounces
db $B4,$B4,$B4			; same height each time
db $10,$10,$10			; same X speed each time

BounceSetting3:			; bounce setting 3
db $03				; 3 bounces
db $C4,$B4,$A4			; different heights
db $14,$10,$0C			; different X speeds


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; init routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

print "INIT ",pc
PHB
PHK
PLB
JSR InitBounceSpeed
PLB
RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; main routine wrapper
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

print "MAIN ",pc
PHB
PHK
PLB
JSR DeadweightMain
PLB
RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; main routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

DeadweightMain:

; typical sprite stuff
JSR DeadweightGFX

LDA $14C8,x
CMP #$08
BNE .Return
LDA $9D
BNE .Return

JSR SubOffscreenX0

; more specific code starts here
JSL $01A7DC		; interact with the player

LDA $1686,x		; if the sprite no longer interacts with objects...
BMI .FallThrough		; skip the ground check and just update its position

LDA $1588,x		;
PHA				; current ground status

JSL $01802A		; update sprite position (with gravity)

LDA $1588,x		;
AND #$04			; if the sprite is not on the ground now...
BEQ .Return2		; don't shake the ground, set speeds, or increment the bounce counter

JSR SetBounceSpeed	; if the sprite is on the ground, make it bounce

LDA.b #!ImpactSFX>>16	; play a sound effect
STA !ImpactSFX&$FFFF	;

PLA				;
AND #$04			; if the sprite was in the air before...
BNE .Return			;

LDY #!TimeToShake	; shake the ground
STY $1887			;

LDA $77			;
AND #$04			; if the player is on the ground at the moment of impact...
BEQ .Return			;

STY $18BD			; stun the player as well

.Return			;
RTS				;

.FallThrough		;
JSL $01802A		;
RTS				;

.Return2			;
PLA				;
RTS				;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; bounce speed routines
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;--------------------------------
; initialize the bounce counter
;--------------------------------

InitBounceSpeed:

LDA $C2,x			; bounce info table index
ASL				; x2
TAY				; into Y
REP #$20			; 16-bit A
LDA BounceInfoPtrs,y	; load a pointer from the table
STA $08			; $08 = pointer to bounce speeds
SEP #$20			; 8-bit A
LDA ($08)			; first byte of the table is how many times the sprite will bounce
STA $187B,x		;
; I commented this out because although it would work, it is probably unnecessary;
; it is probably better to set the initial XY speed in the spawning routine, since at this point,
; the sprite has not bounced yet.
;LDY #$01			;
;LDA ($08),y			; second byte of the table...
;STA $AA,x			; is the sprite's initial Y speed
;TYA				;
;CLC				;
;ADC $187B,x		; [X+1]th byte of the table, where X is the number of bounces...
;TAY				;
;LDA ($08),y			; will be the sprite's initial X speed
;LDY $157C,x		; if the sprite is going to the left...
;BEQ .NoFlip			;
;EOR #$FF			; then the X speed value is flipped
;INC				;
;.NoFlip			;
;STA $B6,x			; initial X speed
RTS				;

;----------------------------------------------------------------
; set the sprite's XY speed when it bounces and update the bounce counter
;----------------------------------------------------------------

SetBounceSpeed:

LDA $C2,x			; bounce info table index
ASL				; x2
TAY				; into Y
REP #$20			; 16-bit A
LDA BounceInfoPtrs,y	; load a pointer from the table
STA $08			; $08 = pointer to bounce speeds
SEP #$20			; 8-bit A

LDY $160E,x		; bounce speed counter
INY				; +1, since the first speed table begins at the second table value
LDA ($08),y			;
STA $AA,x			; set the sprite's Y speed
TYA				;
CLC				;
ADC $187B,x		; add the number of bounces to get to the second speed table
TAY				;
LDA ($08),y			; X speed value
LDY $157C,x		; if the sprite is moving to the left...
BEQ .NoFlip			;
EOR #$FF			; then the X speed value is flipped
INC				;
.NoFlip			;
STA $B6,x			; set the sprite's X speed

INC $160E,x		; increment the bounce counter

LDA $160E,x		;
CMP $187B,x		; if the sprite has already bounced as many times as it's supposed to...
BCC .Return			;

LDA $1686,x		;
ORA #$80			; then disable its object interaction
STA $1686,x		;

.Return			;
RTS				;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; graphics routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

DeadweightGFX:

JSR GetDrawInfo

LDA $00
STA $0300,y

LDA $01
STA $0301,y

LDA #!WeightTileNum
STA $0302,y

LDA $15F6,x
ORA $64
STA $0303,y

LDY #$02
LDA #$00
JSL $01B7B3
RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; miscellaneous standard subroutines
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Table1:              db $0C,$1C
Table2:              db $01,$02
Table3:              db $40,$B0
Table6:              db $01,$FF
Table4:              db $30,$C0,$A0,$C0,$A0,$F0,$60,$90,$30,$C0,$A0,$80,$A0,$40,$60,$B0
Table5:              db $01,$FF,$01,$FF,$01,$FF,$01,$FF,$01,$FF,$01,$FF,$01,$00,$01,$FF

SubOffscreenX0:
LDA #$00
;BRA SubOffscreenMain
;SubOffscreenX1:
;LDA #$02
;BRA SubOffscreenMain
;SubOffscreenX2:
;LDA #$04
;BRA SubOffscreenMain
;SubOffscreenX3:
;LDA #$06
;BRA SubOffscreenMain
;SubOffscreenX4:
;LDA #$08
;BRA SubOffscreenMain
;SubOffscreenX5:
;LDA #$0A
;BRA SubOffscreenMain
;SubOffscreenX6:
;LDA #$0C
;BRA SubOffscreenMain
;SubOffscreenX7:
;LDA #$0E

SubOffscreenMain:

STA $03

JSR SubIsOffscreen
BEQ Return2

LDA $5B
LSR
BCS VerticalLevel
LDA $D8,x
CLC
ADC #$50
LDA $14D4,x
ADC #$00
CMP #$02
BPL EraseSprite
LDA $167A,x
AND #$04
BNE Return2
LDA $13
AND #$01
ORA $03
STA $01
TAY
LDA $1A
CLC
ADC Table4,y
ROL $00
CMP $E4,x
PHP
LDA $1B
LSR $00
ADC Table5,y
PLP
SBC $14E0,x
STA $00
LSR $01
BCC Label20
EOR #$80
STA $00
Label20:
LDA $00
BPL Return2

EraseSprite:
LDA $14C8,x
CMP #$08
BCC KillSprite
LDY $161A,x
CPY #$FF
BEQ KillSprite
LDA #$00
STA $1938,y
KillSprite:
STZ $14C8,x
Return2:
RTS

VerticalLevel:

LDA $167A,x
AND #$04
BNE Return2
LDA $13
LSR
BCS Return2
AND #$01
STA $01
TAY
LDA $1C
CLC
ADC Table3,y
ROL $00
CMP $D8,x
PHP
LDA $1D
LSR $00
ADC Table6,y
PLP
SBC $14D4,x
STA $00
LDY $02
BEQ Label22
EOR #$80
STA $00
Label22:
LDA $00
BPL Return2
BMI EraseSprite

SubIsOffscreen:
LDA $15A0,x
ORA $186C,x
RTS

GetDrawInfo:

STZ $186C,x
STZ $15A0,x
LDA $E4,x
CMP $1A
LDA $14E0,x
SBC $1B
BEQ OnscreenX
INC $15A0,x
OnscreenX:
LDA $14E0,x
XBA
LDA $E4,x
REP #$20
SEC
SBC $1A
CLC
ADC.w #$0040
CMP #$0180
SEP #$20
ROL A
AND #$01
STA $15C4,x
BNE Invalid

LDY #$00
LDA $1662,x
AND #$20
BEQ OnscreenLoop
INY
OnscreenLoop:
LDA $D8,x
CLC
ADC Table1,y
PHP
CMP $1C
ROL $00
PLP
LDA $14D4,x
ADC #$00
LSR $00
SBC $1D
BEQ OnscreenY
LDA $186C,x
ORA Table2,y
STA $186C,x
OnscreenY:
DEY
BPL OnscreenLoop
LDY $15EA,x
LDA $E4,x
SEC
SBC $1A
STA $00
LDA $D8,x
SEC
SBC $1C
STA $01
RTS

Invalid:
PLA
PLA
RTS

SubHorzPos:

LDY #$00
LDA $94
SEC
SBC $E4,x
STA $0F
LDA $95
SBC $14E0,x
BPL $01
INY
RTS


